package com.tuoheng.auth.service;

import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.security.entity.SecurityUser;
import com.tuoheng.common.core.utils.StringUtils;
import com.tuoheng.system.api.entity.User;
import com.tuoheng.system.api.feign.UserClient;
import com.tuoheng.system.api.vo.UserInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

@Service
public class OAuthUserDetailsService implements UserDetailsService {

    @Autowired
    private UserClient userClient;

    /**
     * 获取用户信息
     *
     * @param username 用户名
     * @return
     * @throws UsernameNotFoundException
     */
    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        JsonResult<UserInfo> result = userClient.getInfoByName(username);
        // 检查用户
        checkUser(result, username);
        // 获取用户信息
        UserDetails userDetails = getUserDetails(result);
        return userDetails;
    }

    /**
     * 检查用户信息
     *
     * @param result   数据源
     * @param username 用户名
     */
    private void checkUser(JsonResult result, String username) {
        // 用户信息验证
        if (StringUtils.isNull(result) || StringUtils.isNull(result.getData())) {
//            log.info("登录用户：{} 不存在.", username);
            throw new UsernameNotFoundException("登录用户：" + username + " 不存在");
        }
    }

    /**
     * 获取用户详情信息
     *
     * @param result 结果
     * @return
     */
    private UserDetails getUserDetails(JsonResult<UserInfo> result) {
        UserInfo info = result.getData();
        Set<String> dbAuthsSet = new HashSet<String>();
        if (StringUtils.isNotEmpty(info.getRoles())) {
            // 获取角色
            dbAuthsSet.addAll(info.getRoles());
            dbAuthsSet.addAll(info.getPermissions());
        }

        Collection<? extends GrantedAuthority> authorities = AuthorityUtils
                .createAuthorityList(dbAuthsSet.toArray(new String[0]));
        User user = info.getUser();

//        // 下面是写死，暂时不和数据库交互
//        if(!(("admin").equals(username))){
//            throw new UsernameNotFoundException("the user is not found");
//        }else{
//            String role = "ADMIN_ROLE";
//            List<SimpleGrantedAuthority> authorities = new ArrayList<>();
//            authorities.add(new SimpleGrantedAuthority(role));
//            String password = passwordEncoder.encode("123456");
//            return new User(username,password,authorities);
//        }

        return new SecurityUser(user.getId(), user.getUsername(), user.getPassword(), true, true, true, true,
                authorities);
    }
}
