package com.tuoheng.gateway.config;

import com.tuoheng.gateway.constant.RSAConstant;
import com.tuoheng.gateway.utils.FilterUtils;
import com.tuoheng.gateway.utils.RSAUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.cloud.gateway.filter.factory.rewrite.CachedBodyOutputMessage;
import org.springframework.cloud.gateway.support.BodyInserterContext;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ReactiveHttpOutputMessage;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpRequestDecorator;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.reactive.function.BodyInserter;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.server.HandlerStrategies;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.lang.reflect.Field;
import java.net.URI;

/**
 * 网关全局过滤器
 */
@Configuration
public class GatewayFilterConfig implements GlobalFilter, Ordered {

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 如果是登录不校验Token
        String requestUrl = exchange.getRequest().getPath().value();
        AntPathMatcher pathMatcher = new AntPathMatcher();
        // 非登录请求统一需要认证
        if (!pathMatcher.match("/api/auth/oauth/**", requestUrl)) {
            String token = exchange.getRequest().getHeaders().getFirst("Authorization");
            if (StringUtils.isBlank(token)) {
                return FilterUtils.invalidToken(exchange);
            }
        }
//        // 修改请求参数,并获取请求参数
//        try {
//            updateRequestParam(exchange);
//        } catch (Exception e) {
//            return FilterUtils.invalidUrl(exchange);
//        }
//        // 获取请求体,修改请求体
//        ServerRequest serverRequest = ServerRequest.create(exchange, HandlerStrategies.withDefaults().messageReaders());
//        Mono<String> modifiedBody = serverRequest.bodyToMono(String.class).flatMap(body -> {
//            String encrypt = RSAUtils.decrypt(body, RSAConstant.PRIVATE_KEY);
//            return Mono.just(encrypt);
//        });
//
//        // 创建BodyInserter修改请求体
//        BodyInserter<Mono<String>, ReactiveHttpOutputMessage> bodyInserter = BodyInserters.fromPublisher(modifiedBody, String.class);
//        HttpHeaders headers = new HttpHeaders();
//        headers.putAll(exchange.getRequest().getHeaders());
//        headers.remove(HttpHeaders.CONTENT_LENGTH);
//        // 创建CachedBodyOutputMessage并且把请求param加入
//        CachedBodyOutputMessage outputMessage = new CachedBodyOutputMessage(exchange, headers);
//        return bodyInserter.insert(outputMessage, new BodyInserterContext()).then(Mono.defer(() -> {
//            ServerHttpRequestDecorator decorator = new ServerHttpRequestDecorator(exchange.getRequest()) {
//                @Override
//                public Flux<DataBuffer> getBody() {
//                    return outputMessage.getBody();
//                }
//            };
//            return chain.filter(exchange.mutate().request(decorator).build());
//        }));
        // 放行
        return chain.filter(exchange);
    }

    /**
     * 修改前端传的参数
     */
    private void updateRequestParam(ServerWebExchange exchange) throws NoSuchFieldException, IllegalAccessException {
        ServerHttpRequest request = exchange.getRequest();
        URI uri = request.getURI();
        String query = uri.getQuery();
        if (StringUtils.isNotBlank(query) && query.contains("param")) {
            String[] split = query.split("=");
            String param = RSAUtils.decrypt(split[1], RSAConstant.PRIVATE_KEY);
            Field targetQuery = uri.getClass().getDeclaredField("query");
            targetQuery.setAccessible(true);
            targetQuery.set(uri, param);
        }
    }

    /**
     * 这是Ordered接口的中的方法
     * 过滤器有一个优先级的问题，这个值越小，优先级越高
     *
     * @return
     */
    @Override
    public int getOrder() {
        return 0;
    }
}
